"""
Tinman Test Application

"""
__author__ = 'Gavin M. Roy'
__email__ = 'gmr@myyearbook.com'
__since__ = '2011-06-06'

import logging
import tinman
from tornado import web

CONFIG = {'Application': {'debug': True,
                          'xsrf_cookies': False},
         'HTTPServer': {'no_keep_alive': False,
                        'ports': [8000],
                        'xheaders': False},
         'Logging': {'filename': 'log.txt',
                     'format': "%(module)-12s# %(lineno)-5d %(levelname) -10s\
%(asctime)s  %(message)s",
                     'level': logging.DEBUG},
         'Routes': [("/", "tinman.test.DefaultHandler")]}


class DefaultHandler(web.RequestHandler):

    def get(self):

        # Send a JSON string for our test
        self.write({"message": "Hello World",
                    "request": {"method": self.request.method,
                                "protocol": self.request.protocol,
                                "path": self.request.path,
                                "query": self.request.query,
                                "remote_ip": self.request.remote_ip,
                                "version": self.request.version},
                    "tinman": {"version":  "%i.%i.%i" % tinman.__version__}})



"""
Tinman Cache Module
"""
__author__ = "Gavin M. Roy"
__email__ = "gmr@myyearbook.com"
__date__ = "2010-03-10"
__version__ = 0.2

from functools import wraps
from logging import debug

# Module wide dictionary to hold the cached values in
local_cache = dict()


def memoize_write(*args):

    # Append the value if the key exists, otherwise just set it
    if args[0].tinman_memoize_key in local_cache:
        debug('memoize append: %s' % key)
        local_cache[args[0].tinman_memoize_key] += args[1]
    else:
        debug('memoize set: %s' % key)
        local_cache[args[0].tinman_memoize_key] = args[1]

    # Call the monkey patched RequestHandler.write
    args[0]._write(args[1])


def memoize_finish(*args):

    # If they passed in a last chunk, run the write
    if len(args) > 1:
        memoize_write(args)

    # Un-Monkey-patch
    args[0].write = args[0]._write
    args[0].finish = args[0]._finish

    # Remove the monkey patched attributes
    del args[0]._write
    del args[0]._finish

    # Call the RequestHandler.finish
    args[0].finish()


# Cache Decorator
def memoize(method):

    @wraps(method)
    def wrapper(*args, **kwargs):

        # Our module wide local_cache
        global local_cache

        if not hasattr(args[0], 'write'):
            raise AttributeError("Could not find the ")

        # Get the class name for the key
        key = repr(args[0])

        # Add the arguments to the key
        for value in args[1:]:
            key += ':%s' % str(value)

        debug('memoize: %s' % key)

        # See if the key is in cache and if so, send it
        if key in local_cache:
            debug('memoize hit: %s' % key)
            return self.finish(local_cache[key])

        # Assign our key
        args[0].tinman_memoize_key = key

        # Monkey-patch the write and finish functions
        args[0]._write = args[0].write
        args[0]._finish = args[0].finish
        args[0].write = memoize_write
        args[0].finish = memoize_finish

        # Return the value
        return method(*args, **kwargs)

    return wrapper


def flush():
    """
    Flush all of the attributes in the cache
    """
    global local_cache
    local_cache = dict()



"""
Tinman command line interface
"""
__author__ = 'Gavin M. Roy'
__email__ = 'gmr@myyearbook.com'
__since__ = '2011-06-06'

# Tinman imports
from . import application
from . import utils
from . import __version__

# General imports
import logging
import multiprocessing
import optparse
import os
import signal
import sys
import time

# Tornado imports
from tornado import httpserver
from tornado import ioloop
from tornado import version as tornado_version


class TinmanConnections(object):
    pass


class TinmanProcess(object):
    """
    Manages the tinman master process when run from the cli
    """

    def __init__(self):

        # Set a default IOloop to None
        self._ioloop = None

        # Get our logger
        self._logger = logging.getLogger('tinman')

    def _build_connections(self, config):
        """Build and attach our supported connections to our IOLoop and
         application object.

        Full configuration file
        """
        # Create a tinman object attribute for access to connections
        setattr(self._application, 'tinman', TinmanConnections())

        # Build a RabbitMQ connection if it exists
        self._build_rabbitmq_connection(config.get('RabbitMQ', None))

    def _build_rabbitmq_connection(self, config):
        """Create a connection to RabbitMQ if we have it configured in our
        configuration file.

        :param config: RabbitMQ node of the configuration file dictionary
        :type config: dict
        """
        if not config:
            return

        # Import RabbitMQ only if we need it
        from . import rabbitmq

        # Create our RabbitMQ instance, it will auto-connect and setup based on
        # this
        rabbitmq = rabbitmq.RabbitMQ(config.get('host', None),
                                  config.get('port', None),
                                  config.get('virtual_host', None),
                                  config.get('username', None),
                                  config.get('password', None))
        setattr(self._application.tinman, 'rabbitmq', rabbitmq)

    def _check_required_configuration_parameters(self, config, options):
        """Validates that the required configuration parameters are set.

        :raises: AttributeError

        """
        # Required sections
        if 'Application' not in config:
            raise AttributeError("Missing Application section in configuration")

        if 'HTTPServer' not in config:
            raise AttributeError("Missing HTTPServer section in configuration")

        if 'Logging' not in config:
            raise AttributeError("Missing Logging section in configuration")

        if not options.route_decorator and 'Routes' not in config:
            raise AttributeError("Missing Routes section in configuration")

        if not options.route_decorator and \
           not isinstance(config['Routes'], list):
            raise AttributeError("Error in Routes section in configuration")

    def _process_options(self):
        """Process the cli options returning the options and arguments"""
        usage = "usage: %prog -c <configfile> [options]"
        version_string = "%%prog %i.%i.%i" % __version__
        description = "Tinman's Tornado application runner"

        # Create our parser and setup our command line options
        parser = optparse.OptionParser(usage=usage,
                                       version=version_string,
                                       description=description)

        parser.add_option("-c", "--config",
                          action="store",
                          dest="config",
                          help="Specify the configuration file for use")
        parser.add_option("-f", "--foreground",
                          action="store_true",
                          dest="foreground",
                          default=False,
                          help="Run interactively in console")
        parser.add_option("-r", "--route-decorator",
                          action="store_true",
                          dest="route_decorator",
                          default=False,
                          help="Utilize the route decorator instead of the Routes\
                                section of the config")

        # Parse our options and arguments
        return parser.parse_args()

    def _shutdown_signal_handler(self, signum, frame):
        """
        Called on SIGTERM to shutdown the sub-process
        """
        if self._ioloop:
            self._ioloop.stop()

    def _start_processes(self, config):

        # Loop through and kick off our processes
        self._children = []
        for port in config['HTTPServer']['ports']:
            self._logger.info("Starting Tinman v%s process for port %i",
                              '%i.%i.%i' % __version__, port)
            # Kick off the child process
            child = multiprocessing.Process(target=self._start_server,
                                            name="tinman-%i" % port,
                                            args=(config, port))
            self._children.append(child)
            child.start()

        # Log our completion
        self._logger.debug("All children spawned")

    def _start_server(self, config, port):

        # Setup our signal handler
        signal.signal(signal.SIGTERM, self._shutdown_signal_handler)

        # Start our application
        self._application = \
            application.TinmanApplication(config.get('Routes', None),
                                          **config.get('Application', dict()))

        # Try and build any connection types we automatically support
        self._build_connections(config)

        # Start the HTTP Server
        self._logger.info("Starting Tornado v%s HTTPServer on port %i",
                          tornado_version, port)
        http_server = httpserver.HTTPServer(self._application)
        http_server.listen(port)

        # Get a handle to the instance of IOLoop
        self._ioloop = ioloop.IOLoop.instance()

        # Start the IOLoop
        try:
            self._ioloop.start()
        except KeyboardInterrupt:
            self._logger.info('KeyboardInterrupt received, shutting down.')

    def run(self):
        """Run the Tinman Process"""

        # Process our command line options
        options, args = self._process_options()

        # No configuration file?
        if options.config is None:
            sys.stdout.write("\nConfiguration not specified, \
running Tinman Test Application\n")
            from . import test
            config = test.CONFIG

        # Load the configuration file
        else:
            config = utils.load_configuration_file(options.config)

        # Check our required options
        self._check_required_configuration_parameters(config, options)

        # If we have a base path set prepend it to our Python import path
        if 'base_path' in config:
            sys.path.insert(0, config['base_path'])

        # Setup our logging
        utils.setup_logging(config['Logging'], options.foreground)

        # Setup our signal handlers
        utils.setup_signals()

        # Daemonize if we need to
        if not options.foreground:
            utils.daemonize(pidfile=config.get("pidfile", None),
                            user=config.get("user", None),
                            group=config.get("group", None))
        else:
            config['Application']['debug'] = True

        self._start_processes(config)

        # Have a main event loop for dealing with stats
        utils.running = True
        while utils.running:
            try:
                time.sleep(1)
            except KeyboardInterrupt:
                self._logger.info("CTRL-C pressed, shutting down.")
                break

        # Tell all the children to shutdown
        for child in self._children:
            self._logger.debug("Sending terminate signal to %s", child.name)
            child.terminate()

        # Loop while the children are shutting down
        self._logger.debug("Waiting for children to shutdown")
        while True in [child.is_alive() for child in self._children]:
            time.sleep(0.5)

        # Remove our pidfile
        if 'pidfile' in config:
            if os.path.exists(config['pidfile']):
                try:
                    os.unlink(config['pidfile'])
                except OSError as e:
                    self._logger.error("Could not remove pidfile: %s", e)

        # Log that we're shutdown cleanly
        self._logger.info("tinman has shutdown")



#!/usr/bin/env python
"""
Core Tinman imports
"""
__author__ = 'Gavin M. Roy'
__email__ = '<gmr@myyearbook.com>'
__since__ = '2011-03-14'
__version__ = 0,2,4

__all__ = ['tinman.application',
           'tinman.cache',
           'tinman.cli',
           'tinman.rabbitmq',
           'tinman.utils',
           'tinman.whitelist']

def main(*args):

    # Import our CLI parser
    from tinman import cli

    # Run the main routine
    process = cli.TinmanProcess()
    process.run()

if __name__ == "__main__":

    # Run our CLI process
    main()



"""
Add RabbitMQ Support by adding a pika client that may be bound to the IO loop
instance

"""
__author__ = 'Gavin M. Roy'
__email__ = 'gmr@myyearbook.com'
__since__ = '2011-06-06'


from . import __version__
import json
import logging
from pika import credentials
from pika import connection
from pika import spec
from pika.adapters import tornado_connection
import time


class RabbitMQ(object):
    """RabbitMQ object to facility easy integration with Pika. Currently
    only supports message publishing.

    """

    DEFAULT_APP_ID = 'tinman'
    DEFAULT_DELIVERY_MODE = 1
    DEFAULT_ENCODING = 'UTF-8'

    def __init__(self, host, port, virtual_host, user, password):
        """Construct our RabbitMQ object for use on the Tornado IOLoop

        :param host: RabbitMQ server host
        :type host: str
        :param port: RabbitMQ server port
        :type port: int
        :param virtual_host: RabbitMQ virtual host to use
        :type virtual_host: str
        :param user: RabbitMQ user to connect as
        :type user: str
        :param password: RabbitMQ user's password
        :type paassword: str

        """

        # Create a logger instance
        self._logger = logging.getLogger('tinman.rabbitmq')

        # We don't have a valid connection until the initial connect is done
        self._connection = None

        # We don't have a channel until we're connected
        self._channel = None

        # Set our app_id for publishing messages
        version = '%i.%i.%i' % __version__
        self.app_id = "%s/%s" % (RabbitMQ.DEFAULT_APP_ID, version)

        # Set our delivery mode for publishing messages
        self.delivery_mode = RabbitMQ.DEFAULT_DELIVERY_MODE

        # Set our encoding for publishing messages
        self.encoding = RabbitMQ.DEFAULT_ENCODING

        # Create our credentials
        creds = credentials.PlainCredentials(username=user, password=password)

        # Create the connection parameters
        self.params = connection.ConnectionParameters(host=host,
                                                      port=port,
                                                      virtual_host=virtual_host,
                                                      credentials=creds)

        # Create a new connection
        tornado_connection.TornadoConnection(self.params, self._on_connected)

    def _on_connected(self, connection):
        """Called when a connection has opened.

        :param connection: RabbitMQ connection for raw communication
        :type connection: pika.connection.Connection

        """
        # Assign our connection to the object
        self._connection = connection

        # Create a channel to communicate on
        self._connection.channel(self._on_channel_opened)

        # Log our connection
        self._logger.info('Connected to RabbitMQ at %s:%i',
                          self.params.host, self.params.port)

    def _on_channel_opened(self, channel):
        """Called when a channel is opened.

        :param channel: RabbitMQ channel for commands and receiving messages
        :type channel: pika.channel.Channel
        """

        # Create a channel to use
        self._channel = channel

        # Log the open channel
        self._logger.info('RabbitMQ channel opened at %s:%i',
                          self.params.host, self.params.port)

    def publish_message(self, exchange, routing_key, message,
                        mimetype='text/plain',
                        mandatory=False,
                        immediate=False):
        """Publish a message to RabbitMQ. Auto-JSON encodes all non-string data
        types.

        :param exchange: RabbitMQ exchange to publish to.
        :type exchange: str
        :param routing_key: RabbitMQ publishing routing key
        :type routing_key: str
        :param mimetime: mimetype of the message
        :type mimetype: str
        :param mandatory: AMQP Basic.Publish mandatory flag
        :type mandatory: bool
        :param immediate: AMQP Basic.Publish immediate flag
        :type immediate: bool
        """

        # Auto-JSON encode if it's not a string
        if not isinstance(message, basestring):
            message = json.dumps(message)
            mimetype = 'application/json'

        # Create the properties for the message
        props = spec.BasicProperties(content_type=mimetype,
                                     content_encoding=self.encoding,
                                     timestamp=time.time(),
                                     app_id=self.app_id,
                                     user_id=self.params.credentials.username)

        # Publish the message
        self._channel.basic_publish(exchange,
                                    routing_key,
                                    message,
                                    props,
                                    mandatory,
                                    immediate)



# -*- coding: utf-8 -*-
"""
Functions used mainly in startup and shutdown of tornado applications
"""
import logging
import os
import os.path
import signal
import sys
import yaml

# Windows doesn't support these
try:
    import grp
except ImportError:
    grp = None
try:
    import pwd
except ImportError:
    pwd = None

from functools import wraps
from socket import gethostname
from tornado.options import enable_pretty_logging

# Callback handlers
rehash_handler = None
shutdown_handler = None

# Application state for shutdown
running = False


def log_method_call(method):
    """
    Logging decorator to send the method and arguments to logging.debug
    """
    @wraps(method)
    def debug_log(*args, **kwargs):

        if logging.getLogger('').getEffectiveLevel() == logging.DEBUG:

            # Get the class name of what was passed to us
            try:
                class_name = args[0].__class__.__name__
            except AttributeError:
                class_name = 'Unknown'
            except IndexError:
                class_name = 'Unknown'

            # Build a list of arguments to send to the logging
            log_args = list()
            for x in xrange(1, len(args)):
                log_args.append(args[x])
            if len(kwargs) > 1:
                log_args.append(kwargs)

            # If we have arguments, log them as well, otherwise just the method
            if log_args:
                logging.debug("%s.%s(%r) Called", class_name, method.__name__,
                             log_args)
            else:
                logging.debug("%s.%s() Called", class_name, method.__name__)

        # Actually execute the method
        return method(*args, **kwargs)

    # Return the debug_log function to the python stack for execution
    return debug_log


def application_name():
    """
    Returns the currently running application name
    """
    return os.path.split(sys.argv[0])[1]


def hostname():
    """
    Returns the hostname for the machine we're running on
    """
    return gethostname().split(".")[0]


def daemonize(pidfile=None, user=None, group=None):
    """
    Fork the Python app into the background and close the appropriate
    "files" to detach from console. Based off of code by Jürgen Hermann and
    http://code.activestate.com/recipes/66012/

    Parameters:

    * pidfile: Pass in a file to write the pid, defaults to
               /tmp/current_process_name-pid_number.pid
    * user: User to run as, defaults to current user
    * group: Group to run as, defaults to current group
    """

    # Flush stdout and stderr
    sys.stdout.flush()
    sys.stderr.flush()

    # Set our default uid, gid
    uid, gid = -1, -1

    # Get the user id if we have a user set
    if pwd and user:
        uid = pwd.getpwnam(user).pw_uid

    # Get the group id if we have a group set
    if grp and group:
        gid = grp.getgrnam(group).gr_gid

    # Fork off from the process that called us
    pid = os.fork()
    if pid > 0:
        sys.exit(0)

    # Second fork to put into daemon mode
    pid = os.fork()
    if pid > 0:
        # exit from second parent, print eventual PID before
        sys.stdout.write('%s: started - PID # %d\n' % (application_name(),
                                                       pid))

        # Setup a pidfile if we weren't passed one
        pidfile = pidfile or \
                  os.path.normpath('/tmp/%s-%i.pid' % (application_name(),
                                                       pid))

        # Write a pidfile out
        with open(pidfile, 'w') as f:
            f.write('%i\n' % pid)

            # If we have uid or gid change the uid/gid for the file
            if uid > -1 or gid > -1:
                os.fchown(f.fileno(), uid, gid)

        # Exit the parent process
        sys.exit(0)

    # Detach from parent environment
    os.chdir(os.path.normpath('/'))
    os.umask(0)
    os.setsid()

    # Redirect stdout, stderr, stdin
    si = file('/dev/null', 'r')
    so = file('/dev/null', 'a+')
    se = file('/dev/null', 'a+', 0)
    os.dup2(si.fileno(), sys.stdin.fileno())
    os.dup2(so.fileno(), sys.stdout.fileno())
    os.dup2(se.fileno(), sys.stderr.fileno())

    # Set the running user
    if user and group:
        logging.info("Changing the running user:group to %s:%s", user, group)
    elif user:
        logging.info("Changing the running user to %s", user)
    elif group:
        logging.info("Changing the group to %s", group)

    # If we have a uid and it's not for the running user
    if uid > -1 and uid != os.geteuid():
            try:
                os.seteuid(uid)
                logging.debug("User changed to %s(%i)", user, uid)
            except OSError as e:
                logging.error("Could not set the user: %s", str(e))

    # if we have a gid and it's not for the current group
    if gid > -1 and gid != os.getegid():
        try:
            os.setgid(gid)
            logging.debug("Process group changed to %s(%i)", group, gid)
        except OSError as e:
            logging.error("Could not set the group: %s", str(e))

    return True


def load_configuration_file(config_file):
    """
    Load our YAML configuration file from disk or error out
    if not found or parsable
    """
    try:
        with file(config_file, 'r') as f:
            config = yaml.load(f)

    except IOError:
        sys.stderr.write('Configuration file not found "%s"\n' % config_file)
        sys.exit(1)

    except yaml.scanner.ScannerError as err:
        sys.stderr.write('Invalid configuration file "%s":\n%s\n' % \
                         (config_file, err))
        sys.exit(1)

    return config


def setup_logging(config, debug=False):
    """
    Setup the logging module to respect our configuration values.
    Expects a dictionary called config with the following parameters

    * directory:   Optional log file output directory
    * filename:    Optional filename, not needed for syslog
    * format:      Format for non-debug mode
    * level:       One of debug, error, warning, info
    * handler:     Optional handler
    * syslog:      If handler == syslog, parameters for syslog
      * address:   Syslog address
      * facility:  Syslog facility

    Passing in debug=True will disable any log output to anything but stdout
    and will set the log level to debug regardless of the config.
    """
    # Set logging levels dictionary
    logging_levels = {'debug':    logging.DEBUG,
                      'info':     logging.INFO,
                      'warning':  logging.WARNING,
                      'error':    logging.ERROR,
                      'critical': logging.CRITICAL}

    # Get the logging value from the dictionary
    logging_level = config['level']

    if debug:
        # Override the logging level to use debug mode
        config['level'] = logging.DEBUG

        # If we have specified a file, remove it so logging info goes to stdout
        if 'filename' in config:
            del config['filename']
    else:
        # Use the configuration option for logging
        config['level'] = logging_levels.get(config['level'], logging.NOTSET)

    # Create our formatter
    if 'format' in config:
        formatter = logging.Formatter(config['format'])
    else:
        formattter = None

    # Pass in our logging config
    logging.basicConfig(**config)
    logging.info('Log level set to %s' % logging_level)

    # Get the default logger
    default_logging = logging.getLogger()

    # Remove the default stream handler
    stream_handler = None
    for handler in default_logging.handlers:
        if isinstance(handler, logging.StreamHandler):
            stream_handler = handler
            break

    # If we have loggers, set their level as well
    loggers = config.get('loggers', list())
    for logger_name in loggers:
        if isinstance(logger_name, str):
            logger = logging.getLogger(logger_name)
            #logger.setLevel(config['level'])
        elif isinstance(logger_name, list):
            logger = logging.getLogger(logger_name[0])
            logger.setLevel(logging_levels.get(logger_name[1], logging.NOTSET))
        else:
            raise ValueError('Wrong data type for logger_name: %s',
                             type(logger_name))

    # Use colorized output
    if config['level'] == logging.DEBUG:
        enable_pretty_logging()

    # If we have supported handler
    elif 'handler' in config:

        # If we want to syslog
        if config['handler'] == 'syslog':

            facility = config['syslog']['facility']
            import logging.handlers as handlers

            # If we didn't type in the facility name
            if facility in handlers.SysLogHandler.facility_names:

                # Create the syslog handler
                address = config['syslog']['address']
                facility = handlers.SysLogHandler.facility_names[facility]
                syslog = handlers.SysLogHandler(address=address,
                                                facility=facility)

                # Set our formatter
                syslog.setFormatter(formatter)

                # Set the level
                syslog.setLevel(config['level'])

                # Add the handler
                default_logging.addHandler(syslog)

                # Remove the StreamHandler
                if stream_handler:
                    default_logging.removeHandler(stream_handler)
            else:
                logging.error('%s:Invalid facility, syslog logging aborted',
                              application_name())


def shutdown():
    """
    Cleanly shutdown the application
    """
    global running

    # Tell all our children to stop
    if shutdown_handler:
        shutdown_handler()

    # Set the running state
    running = False


def setup_signals():
    """
    Setup the signals we want to be notified on
    """
    signal.signal(signal.SIGTERM, _shutdown_signal_handler)
    try:
        signal.signal(signal.SIGHUP, _rehash_signal_handler)
    except AttributeError:
        pass

def _shutdown_signal_handler(signum, frame):
    """
    Called on SIGTERM to shutdown the application
    """
    logging.info("SIGTERM received, shutting down")
    shutdown()


def _rehash_signal_handler(signum, frame):
    """
    Would be cool to handle this and effect changes in the config
    """
    logging.info("SIGHUP received, rehashing config")
    if rehash_handler:
        rehash_handler()


def import_namespaced_class(path):
    """
    Pass in a string in the format of foo.Bar, foo.bar.Baz, foo.bar.baz.Qux
    and it will return a handle to the class
    """
    # Split up our string containing the import and class
    parts = path.split('.')

    # Build our strings for the import name and the class name
    import_name = '.'.join(parts[0:-1])
    class_name = parts[-1]

    # get the handle to the class for the given import
    class_handle = getattr(__import__(import_name, fromlist=class_name),
                           class_name)

    # Return the class handle
    return class_handle



"""
Main Tinman Application Class
"""
__author__ = 'gmr'
__since__ = '6/6/11'

# Import our version number
from . import utils
from . import __version__

import logging
import os

from tornado import web

class TinmanApplication(web.Application):

    def __init__(self, routes=None, **settings):

        # Define our logger
        self._logger = logging.getLogger('tinman')

        if not isinstance(routes, list):
            raise ValueError("Routes parameter must be a list of tuples")

        # Our prepared_routes is what we pass in to Tornado
        prepared_routes = list()

        # Iterate through the routes
        for parts in routes:

            # Prepare the route
            route = self._prepare_route(parts)
            if route:
               # Append our prepared_routes list
                self._logger.info('Appending handler: %r', route)
                prepared_routes.append(route)
            else:
                self._logger.warn('Skipping route %r due to prepare error',
                                  parts)

            # Set the app version from the version setting in this file
            if 'version' not in settings:
                settings['version'] = "%i.%i.%i" % __version__

            # Set the base path for use inside the app since all code locations
            # are not relative
            if 'base_path' not in settings:
                settings['base_path'] = \
                    os.path.dirname(os.path.realpath(__file__))

            # If we have a static_path
            if 'static_path' in settings:
                # Replace __base_path__ with the path this is running from
                settings['static_path'] =\
                    settings['static_path'].replace('__base_path__',
                                                    settings['base_path'])

        if settings.get('route_decorator', None):
            # @TODO make this work with a route decorator
            del settings['route_decorator']
            raise AttributeError("No defined routes")

        # If we specified the UI modules module we need to import it
        if 'ui_modules' in settings:
            try:
                # Assign the modules to the import
                settings['ui_modules'] = \
                    utils.import_namespaced_class(settings['ui_modules'])
            except ImportError as err:
                self._logger.error("Error importing UI Modules %s: %s",
                                   settings['ui_modules'], err)

        # Create our Application for this process
        web.Application.__init__(self, prepared_routes, **settings)

    def _prepare_route(self, attributes):
        """Take a given inbound list for a route and parse it creating the
        route and importing the class it belongs to.

        :param route: Route attributes
        :type route: list or tuple
        :returns: list of prepared route
        """
        # Validate it's a list or set
        if type(attributes) not in (list, tuple):
            self._logger.error("Invalid route, must be a list or tuple: %r",
                               attributes)
            return False

        # By default we do not have extra kwargs
        kwargs = None

        # If we have a regex based route, set it up with a raw string
        if attributes[0] == 're':
            route =r"%s" % attributes[1]
            module = attributes[2]
            if len(attributes) == 4:
                kwargs = attributes[3]
        else:
            route  = r"%s" % attributes[0]
            module = attributes[1]
            if len(attributes) == 3:
                kwargs = attributes[2]

        logging.debug("Initializing route: %s with %s", route, module)

        # Return the reference to the python class at the end of the
        # namespace. eg foo.Baz, foo.bar.Baz
        try:
            handler = utils.import_namespaced_class(module)
        except ImportError as err:
            self._logger.error("Module import error for %s: %r",
                               module, err)
            return None

        # Our base prepared route
        prepared_route = [route, handler]

        # If the route has an optional kwargs dict
        if kwargs:
            prepared_route.append(kwargs)

        # Return the route
        return tuple(prepared_route)



from setuptools import setup

long_description = """\
Tinman is an take what you need package designed to speed development of
Tornado applications.  It includes an application wrapper and a toolbox of
decorators and utilities.
"""
setup(name='tinman',
      version='0.2.4',
      description="Tornado application wrapper and toolbox for \
Tornado development",
      long_description=long_description,
      classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
      ],
      keywords='tornado',
      author='Gavin M. Roy',
      author_email='gmr@myyearbook.com',
      url='http://github.com/gmr/tinman',
      license='BSD',
      packages=['tinman'],
      requires=['ipaddr', 'pyyaml'],
      entry_points=dict(console_scripts=['tinman=tinman:main']),
      zip_safe=True)



"""
Tinman Whitelist Module
"""
__author__ = "Gavin M. Roy"
__email__ = "gmr@myyearbook.com"
__date__ = "2011-03-13"
__version__ = 0.1

from functools import wraps
from ipaddr import IPv4Network, IPv4Address
from tornado.web import HTTPError
from types import FunctionType

def whitelisted(argument=None):
    """
    Decorates a method requiring that the requesting IP address is whitelisted

    Requires a whitelist value as a list in the Application.settings dictionary
    Ip addresses can be an individual IP address or a subnet.

    Examples:
        ['10.0.0.0/8','192.168.1.0/24', '1.2.3.4/32']
    """
    def is_whitelisted(remote_ip, whitelist):

        # Convert the ip into a long int version of the ip address
        user_ip = IPv4Address(remote_ip)

        # Loop through the ranges in the whitelist and check
        if any([user_ip in IPv4Network(entry) for entry in whitelist]):
            return True

        return False

    # If the argument is a function then there were no parameters
    if type(argument) is FunctionType:

        def wrapper(self, *args, **kwargs):
            """
            Check the whitelist against our application.settings dictionary
            whitelist key
            """
            # Validate we have a configured whitelist
            if 'whitelist' not in self.application.settings:
                raise ValueError("whitelist not found in Application.settings")

            if is_whitelisted(self.request.remote_ip,
                              self.application.settings['whitelist']):

                # Call the original function, IP is whitelisted
                return argument(self, *args, **kwargs)

            # The ip address was not in the whitelist
            raise HTTPError(403)

        return wrapper

    # They passed in string or list?
    else:
        # Convert a single ip address to a list
        if isinstance(argument, str):
            argument = [argument]

        # Make sure it's a list
        elif not isinstance(argument, list):
            err = "whitelisted requires no parameters or a string or list"
            raise ValueError(err)

        def argument_wrapper(method):

            def validate(self, *args, **kwargs):
                """
                Validate the ip address agross the list of ip addresses
                passed in as a list
                """
                if is_whitelisted(self.request.remote_ip, argument):

                    # Call the original function, IP is whitelisted
                    return method(self, *args, **kwargs)

                # The ip address was not in the whitelist
                raise HTTPError(403)

            return validate

        return argument_wrapper
